@@ -24,6 +24,30 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: httpsvc.cxx,v $
+ * Revision 1.96  2005/11/30 12:47:41  csoutheren
+ * Removed tabs, reformatted some code, and changed tags for Doxygen
+ *
+ * Revision 1.95  2004/04/03 08:22:20  csoutheren
+ * Remove pseudo-RTTI and replaced with real RTTI
+ *
+ * Revision 1.94  2004/04/03 06:54:24  rjongbloed
+ * Many and various changes to support new Visual C++ 2003
+ *
+ * Revision 1.93  2004/03/23 04:41:05  csoutheren
+ * Fixed compile problem on Linux
+ *
+ * Revision 1.92  2004/03/23 03:40:57  csoutheren
+ * Change service process default to be more useful in some environments
+ *
+ * Revision 1.91  2004/01/17 17:44:54  csoutheren
+ * Changed to use PString::MakeEmpty
+ *
+ * Revision 1.90  2003/09/17 09:02:13  csoutheren
+ * Removed memory leak detection code
+ *
+ * Revision 1.89  2003/02/19 07:23:45  robertj
+ * Changes to allow for single threaded HTTP service processes.
+ *
  * Revision 1.88  2002/11/06 22:47:25  robertj
  * Fixed header comment (copyright etc)
  *
@@ -330,9 +354,9 @@ class PServiceMacros_list : public PServiceMacros_base
 #define new PNEW
 
 
-#define HOME_PAGE 	"http://www.equival.com"
-#define EMAIL     	"equival@equival.com.au"
-#define	EQUIVALENCE	"Equivalence Pty. Ltd."
+#define HOME_PAGE   "http://www.equival.com"
+#define EMAIL       "equival@equival.com.au"
+#define EQUIVALENCE "Equivalence Pty. Ltd."
 
 
 static const PTime ImmediateExpiryTime(0, 0, 0, 1, 1, 1980);
@@ -352,9 +376,9 @@ PHTTPServiceProcess::PHTTPServiceProcess(const Info & inf)
     manufacturersEmail(inf.email != NULL ? inf.email : EMAIL),
     productNameHTML(inf.productHTML != NULL ? inf.productHTML : inf.productName),
     gifHTML(inf.gifHTML),
-    copyrightHolder(inf.copyrightHolder != NULL ? inf.copyrightHolder : EQUIVALENCE),
-    copyrightHomePage(inf.copyrightHomePage != NULL ? inf.copyrightHomePage : HOME_PAGE),
-    copyrightEmail(inf.copyrightEmail != NULL ? inf.copyrightEmail : EMAIL)
+    copyrightHolder(inf.copyrightHolder != NULL ? inf.copyrightHolder : inf.manufacturerName),
+    copyrightHomePage(inf.copyrightHomePage != NULL ? inf.copyrightHomePage : (const char *)manufacturersHomePage),
+    copyrightEmail(inf.copyrightEmail != NULL ? inf.copyrightEmail : (const char *)manufacturersEmail)
 {
   ignoreSignatures = FALSE;
 
@@ -389,7 +413,7 @@ PHTTPServiceProcess::~PHTTPServiceProcess()
 PHTTPServiceProcess & PHTTPServiceProcess::Current() 
 {
   PHTTPServiceProcess & process = (PHTTPServiceProcess &)PProcess::Current();
-  PAssert(process.IsDescendant(PHTTPServiceProcess::Class()), "Not a HTTP service!");
+  PAssert(PIsDescendant(&process, PHTTPServiceProcess), "Not a HTTP service!");
   return process;
 }
 
@@ -465,7 +489,7 @@ BOOL PHTTPServiceProcess::ListenForHTTP(PSocket * listener,
   }
 
   if (stackSize > 1000)
-    new PHTTPServiceThread(stackSize, *this, *httpListeningSocket);
+    new PHTTPServiceThread(stackSize, *this);
 
   return TRUE;
 }
@@ -563,6 +587,56 @@ void PHTTPServiceProcess::GetPageHeader(PHTML & html, const PString & title)
 }
 
 
+PTCPSocket * PHTTPServiceProcess::AcceptHTTP()
+{
+  if (httpListeningSocket == NULL)
+    return NULL;
+
+  if (!httpListeningSocket->IsOpen())
+    return NULL;
+
+  // get a socket when a client connects
+  PTCPSocket * socket = new PTCPSocket;
+  if (socket->Accept(*httpListeningSocket))
+    return socket;
+
+  if (socket->GetErrorCode() != PChannel::Interrupted)
+    PSYSTEMLOG(Error, "Accept failed for HTTP: " << socket->GetErrorText());
+
+  if (httpListeningSocket != NULL && httpListeningSocket->IsOpen())
+    return socket;
+
+  delete socket;
+  return NULL;
+}
+
+
+BOOL PHTTPServiceProcess::ProcessHTTP(PTCPSocket & socket)
+{
+  if (!socket.IsOpen())
+    return TRUE;
+
+  PHTTPServer * server = CreateHTTPServer(socket);
+  if (server == NULL) {
+    PSYSTEMLOG(Error, "HTTP server creation/open failed.");
+    return TRUE;
+  }
+
+  // process requests
+  while (server->ProcessCommand())
+    ;
+
+  // always close after the response has been sent
+  delete server;
+
+  // if a restart was requested, then do it, but only if we are not shutting down
+  if (httpListeningSocket->IsOpen())
+    CompleteRestartSystem();
+
+  return TRUE;
+}
+
+
 void PHTTPServiceProcess::BeginRestartSystem()
 {
   if (restartThread == NULL) {
@@ -624,25 +698,26 @@ PHTTPServer * PHTTPServiceProcess::CreateHTTPServer(PTCPSocket & socket)
   return NULL;
 }
 
+
 PHTTPServer * PHTTPServiceProcess::OnCreateHTTPServer(const PHTTPSpace & httpNameSpace)
 {
   return new PHTTPServer(httpNameSpace);
 }
 
+
 //////////////////////////////////////////////////////////////
 
 PHTTPServiceThread::PHTTPServiceThread(PINDEX stackSize,
-                                       PHTTPServiceProcess & app,
-                                       PSocket & listeningSocket)
+                                       PHTTPServiceProcess & app)
   : PThread(stackSize, AutoDeleteThread, NormalPriority, "HTTP Service:%x"),
-    process(app),
-    listener(listeningSocket)
+    process(app)
 {
   process.httpThreadsMutex.Wait();
   process.httpThreads.Append(this);
   process.httpThreadsMutex.Signal();
 
   myStackSize = stackSize;
+  socket = NULL;
   Resume();
 }
 
@@ -652,41 +727,24 @@ PHTTPServiceThread::~PHTTPServiceThread()
   process.httpThreadsMutex.Wait();
   process.httpThreads.Remove(this);
   process.httpThreadsMutex.Signal();
+  delete socket;
+}
+
+
+void PHTTPServiceThread::Close()
+{
+  if (socket != NULL)
+    socket->Close();
 }
 
 
 void PHTTPServiceThread::Main()
 {
-  if (!listener.IsOpen())
-    return;
-
-  // get a socket when a client connects
-  if (!socket.Accept(listener)) {
-    if (socket.GetErrorCode() != PChannel::Interrupted)
-      PSYSTEMLOG(Error, "Accept failed for HTTP: " << socket.GetErrorText());
-    if (listener.IsOpen())
-      new PHTTPServiceThread(myStackSize, process, listener);
-    return;
+  PTCPSocket * socket = process.AcceptHTTP();
+  if (socket != NULL) {
+    new PHTTPServiceThread(myStackSize, process);
+    process.ProcessHTTP(*socket);
   }
-
-  new PHTTPServiceThread(myStackSize, process, listener);
-
-  PHTTPServer * server = process.CreateHTTPServer(socket);
-  if (server == NULL) {
-    PSYSTEMLOG(Error, "HTTP server creation/open failed.");
-    return;
-  }
-
-  // process requests
-  while (server->ProcessCommand())
-    ;
-
-  // always close after the response has been sent
-  delete server;
-
-  // if a restart was requested, then do it, but only if we are not shutting down
-  if (listener.IsOpen())
-    process.CompleteRestartSystem();
 }
 
 
@@ -1153,15 +1211,16 @@ PServiceHTML::PServiceHTML(const char * title, const char * help, const char * h
 {
   PHTTPServiceProcess::Current().GetPageHeader(*this, title);
 
-  *this << PHTML::Heading(1) << title;
+  ostream & this_stream = *this;
+  this_stream << PHTML::Heading(1) << title;
   
   if (help != NULL)
-    *this << "&nbsp;"
-          << PHTML::HotLink(help)
-          << PHTML::Image(helpGif, "Help", 48, 23, "align=absmiddle")
-          << PHTML::HotLink();
+    this_stream << "&nbsp;"
+                << PHTML::HotLink(help)
+                << PHTML::Image(helpGif, "Help", 48, 23, "align=absmiddle")
+                << PHTML::HotLink();
 
-  *this << PHTML::Heading(1) << PHTML::Paragraph();
+  this_stream << PHTML::Heading(1) << PHTML::Paragraph();
 }
 
 
@@ -1323,7 +1382,7 @@ PServiceMacro::PServiceMacro(const PCaselessString & name, BOOL isBlock)
 
 PObject::Comparison PServiceMacro::Compare(const PObject & obj) const
 {
-  PAssert(obj.IsDescendant(PServiceMacro::Class()), PInvalidCast);
+  PAssert(PIsDescendant(&obj, PServiceMacro), PInvalidCast);
   const PServiceMacro & other = (const PServiceMacro &)obj;
 
   if (isMacroBlock != other.isMacroBlock)
@@ -1738,39 +1797,6 @@ PCREATE_SERVICE_MACRO(SignedInclude,P_EMPTY,args)
   return text;
 }
 
-
-#if PMEMORY_CHECK
-
-PCREATE_SERVICE_MACRO(HeapStatistics,P_EMPTY,P_EMPTY)
-{
-  BOOL oldIgnoreAllocations = PMemoryHeap::SetIgnoreAllocations(TRUE);
-  PStringStream str(5000);
-  PMemoryHeap::SetIgnoreAllocations(oldIgnoreAllocations);
-  PMemoryHeap::DumpStatistics(str);
-  return str;
-}
-
-
-PCREATE_SERVICE_MACRO(HeapDump,request,P_EMPTY)
-{
-  static DWORD lastObjectNumber = 0;
-  DWORD objectNumber = lastObjectNumber;
-  PStringToString vars = request.url.GetQueryVars();
-  if (vars.Contains("object"))
-    objectNumber = vars["object"].AsUnsigned();
-
-  BOOL oldIgnoreAllocations = PMemoryHeap::SetIgnoreAllocations(TRUE);
-  PStringStream str(32000);
-  str << "Dumping objects from #" << objectNumber << '\n';
-  PMemoryHeap::SetIgnoreAllocations(oldIgnoreAllocations);
-  PMemoryHeap::DumpObjectsSince(objectNumber, str);
-  lastObjectNumber = PMemoryHeap::GetAllocationRequest();
-  return str;
-}
-
-#endif
-
-
 PCREATE_SERVICE_MACRO_BLOCK(IfQuery,request,args,block)
 {
   PStringToString vars = request.url.GetQueryVars();
@@ -1836,7 +1862,7 @@ static void SplitCmdAndArgs(const PString & text, PINDEX pos, PCaselessString &
   PINDEX endCmd = macro.FindOneOf(whitespace);
   if (endCmd == P_MAX_INDEX) {
     cmd = macro;
-    args = PString::Empty();
+    args.MakeEmpty();
   }
   else {
     cmd = macro.Left(endCmd);
