@@ -5,7 +5,7 @@
  *
  * Portable Windows Library
  *
- * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
+ * Copyright (c) 1993-2002 Equivalence Pty. Ltd.
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.0 (the "License"); you may not use this file except in
@@ -21,12 +21,15 @@
  *
  * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
  *
- * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
- * All Rights Reserved.
- *
  * Contributor(s): ______________________________________.
  *
  * $Log: telnet.cxx,v $
+ * Revision 1.11  2003/11/13 21:14:57  csoutheren
+ * Added fix for telnet NOP command thanks to Andrei Koulik
+ *
+ * Revision 1.10  2002/11/06 22:47:25  robertj
+ * Fixed header comment (copyright etc)
+ *
  * Revision 1.9  2002/09/18 06:38:59  robertj
  * Fixed initialisation of debug flag, thanks wolfboy@netease.com
  *
@@ -911,6 +914,8 @@ void PTelnetSocket::OnSubOption(BYTE code, const BYTE * info, PINDEX len)
 
 BOOL PTelnetSocket::OnCommand(BYTE code)
 {
+  if (code == NOP)
+    return TRUE;
   PTelnetError << "unknown command " << (int)code << endl;
   return TRUE;
 }
