@@ -27,6 +27,64 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: object.cxx,v $
+ * Revision 1.83  2005/12/12 00:24:26  dereksmithies
+ * Add recursive mutex to Wrapper object, so one can successfully Dump Memory
+ * usage Statistics & objects during program operation.
+ *
+ * Revision 1.82  2005/09/23 15:30:46  dominance
+ * more progress to make mingw compile nicely. Thanks goes to Julien Puydt for pointing out to me how to do it properly. ;)
+ *
+ * Revision 1.81  2004/10/24 03:43:55  rjongbloed
+ * Used correct test for inclusion of pragma implementation
+ *
+ * Revision 1.80  2004/10/23 10:55:40  ykiryanov
+ * Added ifdef _WIN32_WCE for PocketPC 2003 SDK port
+ *
+ * Revision 1.79  2004/07/11 07:56:36  csoutheren
+ * Applied jumbo VxWorks patch, thanks to Eize Slange
+ *
+ * Revision 1.78  2004/07/01 11:41:30  csoutheren
+ * Fixed compile and run problems on Linux
+ *
+ * Revision 1.77  2004/06/30 12:17:06  rjongbloed
+ * Rewrite of plug in system to use single global variable for all factories to avoid all sorts
+ *   of issues with startup orders and Windows DLL multiple instances.
+ *
+ * Revision 1.76  2004/06/29 08:20:46  rogerhardiman
+ * The throws are needed on new and delete in GCC 2.95.x too.
+ * This is now important as other code changes (factory and #include <string>)
+ * now cause /usr/include/g++/new to be included too. Tested on FreeBSD 4.10
+ *
+ * Revision 1.75  2004/06/17 22:02:51  csoutheren
+ * Added check to prevent allocationBreakpoint from causing unintentional break every 2^32 memory allocations
+ *
+ * Revision 1.74  2004/06/01 05:22:43  csoutheren
+ * Restored memory check functionality
+ *
+ * Revision 1.73  2004/04/14 06:58:05  csoutheren
+ * Fixed PAtomicInteger and PSmartPointer to use real atomic operations
+ *
+ * Revision 1.72  2004/04/13 11:47:52  csoutheren
+ * Changed PSmartPtr to use PAtomicInteger
+ *
+ * Revision 1.71  2004/04/03 08:22:21  csoutheren
+ * Remove pseudo-RTTI and replaced with real RTTI
+ *
+ * Revision 1.70  2004/04/03 08:09:16  csoutheren
+ * Fixed compile problem on Windows
+ *
+ * Revision 1.69  2004/04/03 07:41:01  csoutheren
+ * Fixed compile problem with ostringstream/ostrstream
+ *
+ * Revision 1.68  2004/04/03 06:54:28  rjongbloed
+ * Many and various changes to support new Visual C++ 2003
+ *
+ * Revision 1.67  2003/09/17 09:02:14  csoutheren
+ * Removed memory leak detection code
+ *
+ * Revision 1.66  2003/09/17 05:55:10  csoutheren
+ * Removed recursive includes
+ *
  * Revision 1.65  2003/02/11 08:05:55  robertj
  * Fixed problems in dumping memory leaks and statistics as PMemoryHeap
  *   is flagged deleted too early, thanks Diego Trtara
@@ -236,24 +294,43 @@
 //
  */
 
+#ifdef __GNUC__
+#pragma implementation "pfactory.h"
+#endif // __GNUC__
+
 #include <ptlib.h>
 #include <ctype.h>
 #ifdef _WIN32
-#include <strstrea.h>
-#include <ptlib/debstrm.h>
+#include <ptlib/msos/ptlib/debstrm.h>
+#ifdef _MSC_VER
 #include <crtdbg.h>
+#endif
 #elif defined(__NUCLEUS_PLUS__)
 #include <ptlib/NucleusDebstrm.h>
 #else
-#if defined(__GNUC__) && __GNUC__ > 2
-#include <strstream>
-#else
-#include <strstream.h>
-#endif
 #include <signal.h>
 #endif
 
 
+PFactoryBase::FactoryMap & PFactoryBase::GetFactories()
+{
+  static FactoryMap factories;
+  return factories;
+}
+
+PMutex & PFactoryBase::GetFactoriesMutex()
+{
+  static PMutex mutex;
+  return mutex;
+}
+
+PFactoryBase::FactoryMap::~FactoryMap()
+{
+  FactoryMap::const_iterator entry;
+  for (entry = begin(); entry != end(); ++entry)
+    delete entry->second;
+}
+
 void PAssertFunc(const char * file,
                  int line,
                  const char * className,
@@ -306,7 +383,12 @@ void PAssertFunc(const char * file, int line, const char * className, const char
 #endif
 
 #ifndef _WIN32_WCE
+
+  #if (__GNUC__ >= 3) && defined (__USE_STL__)
+  ostringstream str;
+  #else
   ostrstream str;
+  #endif
   str << "Assertion fail: ";
   if (msg != NULL)
     str << msg << ", ";
@@ -316,11 +398,105 @@ void PAssertFunc(const char * file, int line, const char * className, const char
   if (err != 0)
     str << ", Error=" << err;
   str << ends;
+  
+  #if (__GNUC__ >= 3) && defined (__USE_STL__)
+  const char * s = str.str().c_str();
+  #else
+  const char * s = str.str();
+  #endif
+
+  PAssertFunc(s);
 
-  PAssertFunc(str.str());
 #endif // !_WIN32_WCE
 }
 
+PObject::Comparison PObject::CompareObjectMemoryDirect(const PObject&obj) const
+{
+  int retval = memcmp(this, &obj, sizeof(PObject));
+  if (retval < 0)
+    return LessThan;
+  if (retval > 0)
+    return GreaterThan;
+  return EqualTo;
+}
+
+
+PObject * PObject::Clone() const
+{
+  PAssertAlways(PUnimplementedFunction);
+  return NULL;
+}
+
+
+PObject::Comparison PObject::Compare(const PObject & obj) const
+{
+  return (Comparison)CompareObjectMemoryDirect(obj);
+}
+
+
+void PObject::PrintOn(ostream & strm) const
+{
+  strm << GetClass();
+}
+
+
+void PObject::ReadFrom(istream &)
+{
+}
+
+
+PINDEX PObject::HashFunction() const
+{
+  return 0;
+}
+
+
+///////////////////////////////////////////////////////////////////////////////
+// General reference counting support
+
+PSmartPointer::PSmartPointer(const PSmartPointer & ptr)
+{
+  object = ptr.object;
+  if (object != NULL)
+    ++object->referenceCount;
+}
+
+
+PSmartPointer & PSmartPointer::operator=(const PSmartPointer & ptr)
+{
+  if (object == ptr.object)
+    return *this;
+
+  if ((object != NULL) && (--object->referenceCount == 0))
+      delete object;
+
+  object = ptr.object;
+  if (object != NULL)
+    ++object->referenceCount;
+
+  return *this;
+}
+
+
+PSmartPointer::~PSmartPointer()
+{
+  if ((object != NULL) && (--object->referenceCount == 0))
+    delete object;
+}
+
+
+PObject::Comparison PSmartPointer::Compare(const PObject & obj) const
+{
+  PAssert(PIsDescendant(&obj, PSmartPointer), PInvalidCast);
+  PSmartObject * other = ((const PSmartPointer &)obj).object;
+  if (object == other)
+    return EqualTo;
+  return object < other ? LessThan : GreaterThan;
+}
+
+
+
+//////////////////////////////////////////////////////////////////////////////////////////
 
 #if PMEMORY_CHECK
 
@@ -328,8 +504,7 @@ void PAssertFunc(const char * file, int line, const char * className, const char
 #undef realloc
 #undef free
 
-
-#if __GNUC__ >= 3
+#if (__GNUC__ >= 3) || ((__GNUC__ == 2)&&(__GNUC_MINOR__ >= 95)) //2.95.X & 3.X
 void * operator new(size_t nSize) throw (std::bad_alloc)
 #else
 void * operator new(size_t nSize)
@@ -339,7 +514,7 @@ void * operator new(size_t nSize)
 }
 
 
-#if __GNUC__ >= 3
+#if (__GNUC__ >= 3) || ((__GNUC__ == 2)&&(__GNUC_MINOR__ >= 95)) //2.95.X & 3.X
 void * operator new[](size_t nSize) throw (std::bad_alloc)
 #else
 void * operator new[](size_t nSize)
@@ -349,7 +524,7 @@ void * operator new[](size_t nSize)
 }
 
 
-#if __GNUC__ >= 3
+#if (__GNUC__ >= 3) || ((__GNUC__ == 2)&&(__GNUC_MINOR__ >= 95)) //2.95.X & 3.X
 void operator delete(void * ptr) throw()
 #else
 void operator delete(void * ptr)
@@ -359,7 +534,7 @@ void operator delete(void * ptr)
 }
 
 
-#if __GNUC__ >= 3
+#if (__GNUC__ >= 3) || ((__GNUC__ == 2)&&(__GNUC_MINOR__ >= 95)) //2.95.X & 3.X
 void operator delete[](void * ptr) throw()
 #else
 void operator delete[](void * ptr)
@@ -369,7 +544,7 @@ void operator delete[](void * ptr)
 }
 
 
-DWORD PMemoryHeap::allocationBreakpoint;
+DWORD PMemoryHeap::allocationBreakpoint = 0;
 char PMemoryHeap::Header::GuardBytes[NumGuardBytes];
 
 
@@ -389,6 +564,8 @@ PMemoryHeap::Wrapper::Wrapper()
   PAssertOS((err = MPEnterCriticalRegion(instance->mutex, kDurationForever)) == 0);
 #elif defined(P_PTHREADS)
   pthread_mutex_lock(&instance->mutex);
+#elif defined(P_VXWORKS)
+  semTake((SEM_ID)instance->mutex, WAIT_FOREVER);
 #endif
 }
 
@@ -405,6 +582,8 @@ PMemoryHeap::Wrapper::~Wrapper()
   PAssertOS((err = MPExitCriticalRegion(instance->mutex)) == 0 || instance->isDestroyed);
 #elif defined(P_PTHREADS)
   pthread_mutex_unlock(&instance->mutex);
+#elif defined(P_VXWORKS)
+  semGive((SEM_ID)instance->mutex);
 #endif
 }
 
@@ -437,10 +616,15 @@ PMemoryHeap::PMemoryHeap()
   static PDebugStream debug;
   leakDumpStream = &debug;
 #else
-#if defined(P_MAC_MPTHREADS)
-  PAssertOS(MPCreateCriticalRegion(&mutex) == 0);
-#elif defined(P_PTHREADS)
-  pthread_mutex_init(&mutex, NULL);
+#if defined(P_PTHREADS)
+#ifdef PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP
+  pthread_mutex_t recursiveMutex = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
+  mutex = recursiveMutex;
+#else
+ pthread_mutex_init(&mutex, NULL);
+#endif
+#elif defined(P_VXWORKS)
+  mutex = semMCreate(SEM_Q_FIFO);
 #endif
   leakDumpStream = &cerr;
 #endif
@@ -460,10 +644,10 @@ PMemoryHeap::~PMemoryHeap()
   DeleteCriticalSection(&mutex);
   extern void PWaitOnExitConsoleWindow();
   PWaitOnExitConsoleWindow();
-#elif defined(P_MAC_MPTHREADS)
-  MPDeleteCriticalRegion(mutex);
 #elif defined(P_PTHREADS)
   pthread_mutex_destroy(&mutex);
+#elif defined(P_VXWORKS)
+  semDelete((SEM_ID)mutex);
 #endif
 }
 
@@ -508,9 +692,11 @@ void * PMemoryHeap::InternalAllocate(size_t nSize, const char * file, int line,
   if (firstRealObject == 0 && (flags&NoLeakPrint) == 0)
     firstRealObject = allocationRequest;
 
-  if (allocationRequest == allocationBreakpoint) {
+  if (allocationBreakpoint != 0 && allocationRequest == allocationBreakpoint) {
 #ifdef _WIN32
     __asm int 3;
+#elif defined(P_VXWORKS)
+    kill(taskIdSelf(), SIGABRT);
 #else
     kill(getpid(), SIGABRT);
 #endif 
@@ -575,9 +761,11 @@ void * PMemoryHeap::Reallocate(void * ptr, size_t nSize, const char * file, int
     return NULL;
   }
 
-  if (mem->allocationRequest == mem->allocationBreakpoint) {
+  if (mem->allocationBreakpoint != 0 && mem->allocationRequest == mem->allocationBreakpoint) {
 #ifdef _WIN32
     __asm int 3;
+#elif defined(P_VXWORKS)
+    kill(taskIdSelf(), SIGABRT);
 #else
     kill(getpid(), SIGABRT);
 #endif 
@@ -859,7 +1047,7 @@ void PMemoryHeap::InternalDumpObjectsSince(DWORD objectNumber, ostream & strm)
 
 #ifndef P_VXWORKS
 
-#if __GNUC__ >= 3
+#if (__GNUC__ >= 3) || ((__GNUC__ == 2)&&(__GNUC_MINOR__ >= 95)) //2.95.X & 3.X
 void * operator new[](size_t nSize) throw (std::bad_alloc)
 #else
 void * operator new[](size_t nSize)
@@ -868,7 +1056,7 @@ void * operator new[](size_t nSize)
   return malloc(nSize);
 }
 
-#if __GNUC__ >= 3
+#if (__GNUC__ >= 3) || ((__GNUC__ == 2)&&(__GNUC_MINOR__ >= 95)) //2.95.X & 3.X
 void operator delete[](void * ptr) throw ()
 #else
 void operator delete[](void * ptr)
@@ -882,432 +1070,6 @@ void operator delete[](void * ptr)
 #endif // PMEMORY_CHECK
 
 
-const char * PObject::GetClass(unsigned) const
-{
-  return Class();
-}
-
-
-BOOL PObject::IsClass(const char * clsName) const
-{
-  return strcmp(clsName, Class()) == 0;
-}
-
-
-BOOL PObject::IsDescendant(const char * clsName) const
-{
-  return strcmp(clsName, Class()) == 0;
-}
-
-
-PObject::Comparison PObject::CompareObjectMemoryDirect(const PObject&obj) const
-{
-  int retval = memcmp(this, &obj, sizeof(PObject));
-  if (retval < 0)
-    return LessThan;
-  if (retval > 0)
-    return GreaterThan;
-  return EqualTo;
-}
-
-
-PObject * PObject::Clone() const
-{
-  PAssertAlways(PUnimplementedFunction);
-  return NULL;
-}
-
-
-PObject::Comparison PObject::Compare(const PObject & obj) const
-{
-  return (Comparison)CompareObjectMemoryDirect(obj);
-}
-
-
-void PObject::PrintOn(ostream & strm) const
-{
-  strm << GetClass();
-}
-
-
-void PObject::ReadFrom(istream &)
-{
-}
-
-
-PINDEX PObject::PreSerialise(PSerialiser &)
-{
-  return 0;
-}
-
-
-void PObject::Serialise(PSerialiser &)
-{
-}
-
-
-void PObject::UnSerialise(PUnSerialiser &)
-{
-}
-
-
-PINDEX PObject::HashFunction() const
-{
-  return 0;
-}
-
-
-///////////////////////////////////////////////////////////////////////////////
-// Serialisation support
-
-PSerialRegistration * PSerialRegistration::creatorHashTable[
-                                           PSerialRegistration::HashTableSize];
-
-PINDEX PSerialRegistration::HashFunction(const char * className)
-{
-  PINDEX hash = (BYTE)className[0];
-  if (className[0] != '\0') {
-    hash += (BYTE)className[1];
-    if (className[1] != '\0')
-      hash += (BYTE)className[2];
-  }
-  return hash%HashTableSize;
-}
-
-
-PSerialRegistration::PSerialRegistration(const char * clsNam,
-                                                          CreatorFunction func)
-  : className(clsNam), creator(func)
-{
-  clash = NULL;
-
-  PINDEX hash = HashFunction(className);
-  if (creatorHashTable[hash] != NULL)
-    creatorHashTable[hash]->clash = this;
-  creatorHashTable[hash] = this;
-}
-
-
-PSerialRegistration::CreatorFunction
-                           PSerialRegistration::GetCreator(const char * clsNam)
-{
-  PINDEX hash = HashFunction(clsNam);
-  PSerialRegistration * reg = creatorHashTable[hash];
-  while (reg != NULL) {
-    if (strcmp(reg->className, clsNam) == 0)
-      return reg->creator;
-  }
-  return NULL;
-}
-
-
-PSerialiser::PSerialiser(ostream & strm)
-  : stream(strm)
-{
-}
-
-
-PSerialiser & PSerialiser::operator<<(PObject & obj)
-{
-  obj.Serialise(*this);
-  return *this;
-}
-
-
-PTextSerialiser::PTextSerialiser(ostream & strm, PObject & data)
-  : PSerialiser(strm)
-{
-  data.Serialise(*this);
-}
-
-
-PSerialiser & PTextSerialiser::operator<<(char)
-  { return *this; }
-
-PSerialiser & PTextSerialiser::operator<<(unsigned char)
-  { return *this; }
-
-PSerialiser & PTextSerialiser::operator<<(signed char)
-  { return *this; }
-
-PSerialiser & PTextSerialiser::operator<<(short)
-  { return *this; }
-
-PSerialiser & PTextSerialiser::operator<<(unsigned short)
-  { return *this; }
-
-PSerialiser & PTextSerialiser::operator<<(int)
-  { return *this; }
-
-PSerialiser & PTextSerialiser::operator<<(unsigned int)
-  { return *this; }
-
-PSerialiser & PTextSerialiser::operator<<(long)
-  { return *this; }
-
-PSerialiser & PTextSerialiser::operator<<(unsigned long)
-  { return *this; }
-
-PSerialiser & PTextSerialiser::operator<<(float)
-  { return *this; }
-
-PSerialiser & PTextSerialiser::operator<<(double)
-  { return *this; }
-
-#ifndef NO_LONG_DOUBLE
-PSerialiser & PTextSerialiser::operator<<(long double)
-  { return *this; }
-#endif
-
-PSerialiser & PTextSerialiser::operator<<(const char *)
-  { return *this; }
-
-PSerialiser & PTextSerialiser::operator<<(const unsigned char *)
-  { return *this; }
-
-PSerialiser & PTextSerialiser::operator<<(const signed char *)
-  { return *this; }
-
-PSerialiser & PTextSerialiser::operator<<(PObject & obj)
-  { return PSerialiser::operator<<(obj); }
-
-
-PBinarySerialiser::PBinarySerialiser(ostream & strm, PObject & data)
-  : PSerialiser(strm)
-{
-  classesUsed = new PSortedStringList;
-  data.PreSerialise(*this);
-  stream << *classesUsed;
-  data.Serialise(*this);
-}
-
-
-PBinarySerialiser::~PBinarySerialiser()
-{
-  delete classesUsed;
-}
-
-
-PSerialiser & PBinarySerialiser::operator<<(char)
-  { return *this; }
-
-PSerialiser & PBinarySerialiser::operator<<(unsigned char)
-  { return *this; }
-
-PSerialiser & PBinarySerialiser::operator<<(signed char)
-  { return *this; }
-
-PSerialiser & PBinarySerialiser::operator<<(short)
-  { return *this; }
-
-PSerialiser & PBinarySerialiser::operator<<(unsigned short)
-  { return *this; }
-
-PSerialiser & PBinarySerialiser::operator<<(int)
-  { return *this; }
-
-PSerialiser & PBinarySerialiser::operator<<(unsigned int)
-  { return *this; }
-
-PSerialiser & PBinarySerialiser::operator<<(long)
-  { return *this; }
-
-PSerialiser & PBinarySerialiser::operator<<(unsigned long)
-  { return *this; }
-
-PSerialiser & PBinarySerialiser::operator<<(float)
-  { return *this; }
-
-PSerialiser & PBinarySerialiser::operator<<(double)
-  { return *this; }
-
-#ifndef NO_LONG_DOUBLE
-PSerialiser & PBinarySerialiser::operator<<(long double)
-  { return *this; }
-#endif
-
-PSerialiser & PBinarySerialiser::operator<<(const char *)
-  { return *this; }
-
-PSerialiser & PBinarySerialiser::operator<<(const unsigned char *)
-  { return *this; }
-
-PSerialiser & PBinarySerialiser::operator<<(const signed char *)
-  { return *this; }
-
-PSerialiser & PBinarySerialiser::operator<<(PObject & obj)
-  { return PSerialiser::operator<<(obj); }
-
-
-PUnSerialiser::PUnSerialiser(istream & strm)
-  : stream(strm)
-{
-}
-
-
-PTextUnSerialiser::PTextUnSerialiser(istream & strm)
-  : PUnSerialiser(strm)
-{
-}
-
-
-PUnSerialiser & PTextUnSerialiser::operator>>(char &)
-  { return *this; }
-
-PUnSerialiser & PTextUnSerialiser::operator>>(unsigned char &)
-  { return *this; }
-
-PUnSerialiser & PTextUnSerialiser::operator>>(signed char &)
-  { return *this; }
-
-PUnSerialiser & PTextUnSerialiser::operator>>(short &)
-  { return *this; }
-
-PUnSerialiser & PTextUnSerialiser::operator>>(unsigned short &)
-  { return *this; }
-
-PUnSerialiser & PTextUnSerialiser::operator>>(int &)
-  { return *this; }
-
-PUnSerialiser & PTextUnSerialiser::operator>>(unsigned int &)
-  { return *this; }
-
-PUnSerialiser & PTextUnSerialiser::operator>>(long &)
-  { return *this; }
-
-PUnSerialiser & PTextUnSerialiser::operator>>(unsigned long &)
-  { return *this; }
-
-PUnSerialiser & PTextUnSerialiser::operator>>(float &)
-  { return *this; }
-
-PUnSerialiser & PTextUnSerialiser::operator>>(double &)
-  { return *this; }
-
-#ifndef NO_LONG_DOUBLE
-PUnSerialiser & PTextUnSerialiser::operator>>(long double &)
-  { return *this; }
-#endif
-
-PUnSerialiser & PTextUnSerialiser::operator>>(char *)
-  { return *this; }
-
-PUnSerialiser & PTextUnSerialiser::operator>>(unsigned char *)
-  { return *this; }
-
-PUnSerialiser & PTextUnSerialiser::operator>>(signed char *)
-  { return *this; }
-
-PUnSerialiser & PTextUnSerialiser::operator>>(PObject &)
-  { return *this; }
-
-
-PBinaryUnSerialiser::PBinaryUnSerialiser(istream & strm)
-  : PUnSerialiser(strm)
-{
-  classesUsed = new PStringArray;
-}
-
-
-PBinaryUnSerialiser::~PBinaryUnSerialiser()
-{
-  delete classesUsed;
-}
-
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(char &)
-  { return *this; }
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(unsigned char &)
-  { return *this; }
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(signed char &)
-  { return *this; }
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(short &)
-  { return *this; }
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(unsigned short &)
-  { return *this; }
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(int &)
-  { return *this; }
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(unsigned int &)
-  { return *this; }
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(long &)
-  { return *this; }
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(unsigned long &)
-  { return *this; }
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(float &)
-  { return *this; }
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(double &)
-  { return *this; }
-
-#ifndef NO_LONG_DOUBLE
-PUnSerialiser & PBinaryUnSerialiser::operator>>(long double &)
-  { return *this; }
-#endif
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(char *)
-  { return *this; }
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(unsigned char *)
-  { return *this; }
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(signed char *)
-  { return *this; }
-
-PUnSerialiser & PBinaryUnSerialiser::operator>>(PObject &)
-  { return *this; }
-
-
-///////////////////////////////////////////////////////////////////////////////
-// General reference counting support
-
-PSmartPointer::PSmartPointer(const PSmartPointer & ptr)
-{
-  object = ptr.object;
-  if (object != NULL)
-    ++object->referenceCount;
-}
-
-
-PSmartPointer & PSmartPointer::operator=(const PSmartPointer & ptr)
-{
-  if (object == ptr.object)
-    return *this;
-
-  if (object != NULL && --object->referenceCount == 0)
-    delete object;
-
-  object = ptr.object;
-  if (object != NULL)
-    ++object->referenceCount;
-
-  return *this;
-}
-
-
-PSmartPointer::~PSmartPointer()
-{
-  if (object != NULL && --object->referenceCount == 0)
-    delete object;
-}
-
-
-PObject::Comparison PSmartPointer::Compare(const PObject & obj) const
-{
-  PAssert(obj.IsDescendant(PSmartPointer::Class()), PInvalidCast);
-  PSmartObject * other = ((const PSmartPointer &)obj).object;
-  if (object == other)
-    return EqualTo;
-  return object < other ? LessThan : GreaterThan;
-}
 
 
 ///////////////////////////////////////////////////////////////////////////////
